# Generated by Django 3.2.4 on 2022-03-22 14:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ashley", "0005_uploadimage"),
    ]

    operations = [
        migrations.AddField(
            model_name="lticontext",
            name="is_marked_locked",
            field=models.BooleanField(default=False),
        ),
    ]



"""XAPI module."""
import logging
import uuid
from typing import Optional

from django.utils import timezone
from tincan import Activity, Agent, AgentAccount, Context, Statement, Verb

from ashley.models import AbstractUser

logger = logging.getLogger(__name__)


def build_statement(
    user: AbstractUser,
    verb: Verb,
    obj: Activity,
    context: Context,
    statement_id: Optional[uuid.UUID] = None,
) -> Optional[Statement]:
    """Build a XAPI Statement based on the current context"""
    timestamp = timezone.now().isoformat()
    actor = _get_actor_from_user(user)
    if statement_id is None:
        statement_id = uuid.uuid4()
    if actor is None:
        logger.warning("Unable to get an XAPI actor definition for user %s", user.id)
        return None
    return Statement(
        actor=actor,
        context=context,
        id=statement_id,
        object=obj,
        timestamp=timestamp,
        verb=verb,
    )


def _get_actor_from_user(user: AbstractUser) -> Optional[Agent]:
    """Generate a XAPI Agent object from a Ashley User object"""
    if user.lti_remote_user_id and user.lti_consumer.url:
        return Agent(
            account=AgentAccount(
                name=user.lti_remote_user_id, home_page=user.lti_consumer.url
            )
        )
    return None



"""Serializer for ashley api."""
from django.contrib.auth import get_user_model
from machina.core.db.models import get_model
from rest_framework import serializers

User = get_user_model()
UploadImage = get_model("ashley", "UploadImage")


class UserSerializer(serializers.ModelSerializer):
    """Serializer for User model."""

    class Meta:
        model = User
        fields = ["id", "public_username"]
        read_only_fields = ["id", "public_username"]


class UploadImageSerializer(serializers.ModelSerializer):
    """Serializer for Image Upload model."""

    class Meta:
        model = UploadImage
        fields = (
            "id",
            "file",
            "forum",
            "poster",
        )
        extra_kwargs = {"poster": {"required": False}}



"""This module defines signal receivers for the Ashley application"""

import logging

from django.conf import settings
from django.dispatch import receiver
from django.utils.translation import to_locale
from machina.apps.forum.signals import forum_viewed
from machina.apps.forum_conversation.signals import topic_viewed
from machina.conf import settings as machina_settings
from machina.core.db.models import get_model
from tincan import (
    Activity,
    ActivityDefinition,
    Context,
    ContextActivities,
    LanguageMap,
    Verb,
)

from .machina_extensions.forum_conversation.signals import (
    post_created,
    post_updated,
    topic_created,
    topic_updated,
)
from .xapi import build_statement

LTIContext = get_model("ashley", "LTIContext")

logger = logging.getLogger(__name__)


@receiver(forum_viewed)
# pylint: disable=unused-argument
def track_forum_view(sender, forum, user, request, response, **kwargs):
    """Log a XAPI statement when a user views a forum."""

    parent_activities = None

    consumer = getattr(user, "lti_consumer", None)
    if consumer is None:
        logger.warning("Unable to get LTI consumer of user %s", user)
        return
    xapi_logger = logging.getLogger(f"xapi.{user.lti_consumer.slug}")

    verb = Verb(
        id="http://id.tincanapi.com/verb/viewed",
        display=LanguageMap({"en-US": "viewed"}),
    )

    obj = Activity(
        id=f"id://ashley/forum/{forum.pk}",
        definition=ActivityDefinition(
            name=LanguageMap(
                {to_locale(settings.LANGUAGE_CODE).replace("_", "-"): forum.name}
            ),
            type="http://id.tincanapi.com/activitytype/community-site",
        ),
    )

    if request.forum_permission_handler.current_lti_context_id is not None:
        try:
            lti_context = LTIContext.objects.get(
                pk=request.forum_permission_handler.current_lti_context_id
            )
            parent_activities = [
                Activity(
                    id=lti_context.lti_id,
                    definition=ActivityDefinition(
                        type="http://adlnet.gov/expapi/activities/course"
                    ),
                )
            ]

        except LTIContext.DoesNotExist:
            pass

    if parent_activities is not None:
        context = Context(
            context_activities=ContextActivities(parent=parent_activities),
        )
    else:
        context = None

    statement = build_statement(user, verb, obj, context)
    if statement:
        xapi_logger.info(statement.to_json())


@receiver(topic_viewed)
# pylint: disable=unused-argument
def track_topic_view(sender, topic, user, request, response, **kwargs):
    """Log a XAPI statement when a user views a topic."""

    consumer = getattr(user, "lti_consumer", None)
    if consumer is None:
        logger.warning("Unable to get LTI consumer of user %s", user)
        return
    xapi_logger = logging.getLogger(f"xapi.{user.lti_consumer.slug}")

    verb = Verb(
        id="http://id.tincanapi.com/verb/viewed",
        display=LanguageMap({"en-US": "viewed"}),
    )

    obj = Activity(
        id=f"id://ashley/topic/{topic.pk}",
        definition=ActivityDefinition(
            name=LanguageMap(
                {to_locale(settings.LANGUAGE_CODE).replace("_", "-"): topic.subject}
            ),
            type="http://id.tincanapi.com/activitytype/discussion",
            extensions={
                "https://w3id.org/xapi/acrossx/extensions/total-items": topic.posts_count,
                "https://w3id.org/xapi/acrossx/extensions/total-pages": (
                    (topic.posts_count - 1)
                    // machina_settings.TOPIC_POSTS_NUMBER_PER_PAGE
                )
                + 1,
            },
        ),
    )

    parent_activities = [
        Activity(
            id=f"uuid://{topic.forum.lti_id}",
            definition=ActivityDefinition(
                name=LanguageMap(
                    {
                        to_locale(settings.LANGUAGE_CODE).replace(
                            "_", "-"
                        ): topic.forum.name
                    }
                ),
                type="http://id.tincanapi.com/activitytype/community-site",
            ),
        )
    ]

    if request.forum_permission_handler.current_lti_context_id is not None:
        try:
            lti_context = LTIContext.objects.get(
                pk=request.forum_permission_handler.current_lti_context_id
            )
            parent_activities.append(
                Activity(
                    id=lti_context.lti_id,
                    definition=ActivityDefinition(
                        type="http://adlnet.gov/expapi/activities/course"
                    ),
                )
            )

        except LTIContext.DoesNotExist:
            pass

    context = Context(
        context_activities=ContextActivities(parent=parent_activities),
        extensions={
            "http://www.risc-inc.com/annotator/extensions/page": int(
                request.GET.get("page", default=1)
            )
        },
    )

    statement = build_statement(user, verb, obj, context)
    if statement:
        xapi_logger.info(statement.to_json())


@receiver(topic_created)
# pylint: disable=unused-argument
def track_create_topic(sender, topic, user, request, response, **kwargs):
    """Log a XAPI statement when a user creates a topic."""

    consumer = getattr(user, "lti_consumer", None)
    if consumer is None:
        logger.warning("Unable to get LTI consumer of user %s", user)
        return
    xapi_logger = logging.getLogger(f"xapi.{user.lti_consumer.slug}")

    verb = Verb(
        id="https://w3id.org/xapi/dod-isd/verbs/created",
        display=LanguageMap({"en-US": "created"}),
    )

    obj = Activity(
        id=f"id://ashley/topic/{topic.pk}",
        definition=ActivityDefinition(
            name=LanguageMap(
                {to_locale(settings.LANGUAGE_CODE).replace("_", "-"): topic.subject}
            ),
            type="http://id.tincanapi.com/activitytype/discussion",
        ),
    )

    parent_activities = [
        Activity(
            id=f"uuid://{topic.forum.lti_id}",
            definition=ActivityDefinition(
                name=LanguageMap(
                    {
                        to_locale(settings.LANGUAGE_CODE).replace(
                            "_", "-"
                        ): topic.forum.name
                    }
                ),
                type="http://id.tincanapi.com/activitytype/community-site",
            ),
        )
    ]

    if request.forum_permission_handler.current_lti_context_id is not None:
        try:
            lti_context = LTIContext.objects.get(
                pk=request.forum_permission_handler.current_lti_context_id
            )
            parent_activities.append(
                Activity(
                    id=lti_context.lti_id,
                    definition=ActivityDefinition(
                        type="http://adlnet.gov/expapi/activities/course"
                    ),
                )
            )

        except LTIContext.DoesNotExist:
            pass

    context = Context(
        context_activities=ContextActivities(parent=parent_activities),
    )

    statement = build_statement(user, verb, obj, context)
    if statement:
        xapi_logger.info(statement.to_json())


@receiver(topic_updated)
# pylint: disable=unused-argument
def track_update_topic(sender, topic, user, request, response, **kwargs):
    """Log a XAPI statement when a user updates a topic."""

    consumer = getattr(user, "lti_consumer", None)
    if consumer is None:
        logger.warning("Unable to get LTI consumer of user %s", user)
        return
    xapi_logger = logging.getLogger(f"xapi.{user.lti_consumer.slug}")

    verb = Verb(
        id="https://w3id.org/xapi/dod-isd/verbs/updated",
        display=LanguageMap({"en-US": "updated"}),
    )

    obj = Activity(
        id=f"id://ashley/topic/{topic.pk}",
        definition=ActivityDefinition(
            name=LanguageMap(
                {to_locale(settings.LANGUAGE_CODE).replace("_", "-"): topic.subject}
            ),
            type="http://id.tincanapi.com/activitytype/discussion",
        ),
    )

    parent_activities = [
        Activity(
            id=f"uuid://{topic.forum.lti_id}",
            definition=ActivityDefinition(
                name=LanguageMap(
                    {
                        to_locale(settings.LANGUAGE_CODE).replace(
                            "_", "-"
                        ): topic.forum.name
                    }
                ),
                type="http://id.tincanapi.com/activitytype/community-site",
            ),
        )
    ]

    if request.forum_permission_handler.current_lti_context_id is not None:
        try:
            lti_context = LTIContext.objects.get(
                pk=request.forum_permission_handler.current_lti_context_id
            )
            parent_activities.append(
                Activity(
                    id=lti_context.lti_id,
                    definition=ActivityDefinition(
                        type="http://adlnet.gov/expapi/activities/course"
                    ),
                )
            )

        except LTIContext.DoesNotExist:
            pass

    context = Context(
        context_activities=ContextActivities(parent=parent_activities),
    )

    statement = build_statement(user, verb, obj, context)
    if statement:
        xapi_logger.info(statement.to_json())


@receiver(post_created)
# pylint: disable=unused-argument
def track_create_post(sender, post, user, request, response, **kwargs):
    """Log a XAPI statement when a user creates a post."""

    consumer = getattr(user, "lti_consumer", None)
    if consumer is None:
        logger.warning("Unable to get LTI consumer of user %s", user)
        return
    xapi_logger = logging.getLogger(f"xapi.{user.lti_consumer.slug}")

    verb = Verb(
        id="https://w3id.org/xapi/dod-isd/verbs/created",
        display=LanguageMap({"en-US": "created"}),
    )

    obj = Activity(
        id=f"id://ashley/post/{post.pk}",
        definition=ActivityDefinition(
            name=LanguageMap(
                {to_locale(settings.LANGUAGE_CODE).replace("_", "-"): post.subject}
            ),
            type="https://w3id.org/xapi/acrossx/activities/message",
        ),
    )

    parent_activities = [
        Activity(
            id=f"uuid://{post.topic.forum.lti_id}",
            definition=ActivityDefinition(
                name=LanguageMap(
                    {
                        to_locale(settings.LANGUAGE_CODE).replace(
                            "_", "-"
                        ): post.topic.subject
                    }
                ),
                type="http://id.tincanapi.com/activitytype/discussion",
            ),
        )
    ]

    if request.forum_permission_handler.current_lti_context_id is not None:
        try:
            lti_context = LTIContext.objects.get(
                pk=request.forum_permission_handler.current_lti_context_id
            )
            parent_activities.append(
                Activity(
                    id=lti_context.lti_id,
                    definition=ActivityDefinition(
                        type="http://adlnet.gov/expapi/activities/course"
                    ),
                )
            )

        except LTIContext.DoesNotExist:
            pass

    context = Context(
        context_activities=ContextActivities(parent=parent_activities),
    )

    statement = build_statement(user, verb, obj, context)
    if statement:
        xapi_logger.info(statement.to_json())


@receiver(post_updated)
# pylint: disable=unused-argument
def track_update_post(sender, post, user, request, response, **kwargs):
    """Log a XAPI statement when a user updates a post."""

    consumer = getattr(user, "lti_consumer", None)
    if consumer is None:
        logger.warning("Unable to get LTI consumer of user %s", user)
        return
    xapi_logger = logging.getLogger(f"xapi.{user.lti_consumer.slug}")

    verb = Verb(
        id="https://w3id.org/xapi/dod-isd/verbs/updated",
        display=LanguageMap({"en-US": "updated"}),
    )

    obj = Activity(
        id=f"id://ashley/post/{post.pk}",
        definition=ActivityDefinition(
            name=LanguageMap(
                {to_locale(settings.LANGUAGE_CODE).replace("_", "-"): post.subject}
            ),
            type="https://w3id.org/xapi/acrossx/activities/message",
        ),
    )

    parent_activities = [
        Activity(
            id=f"uuid://{post.topic.forum.lti_id}",
            definition=ActivityDefinition(
                name=LanguageMap(
                    {
                        to_locale(settings.LANGUAGE_CODE).replace(
                            "_", "-"
                        ): post.topic.subject
                    }
                ),
                type="http://id.tincanapi.com/activitytype/discussion",
            ),
        )
    ]

    if request.forum_permission_handler.current_lti_context_id is not None:
        try:
            lti_context = LTIContext.objects.get(
                pk=request.forum_permission_handler.current_lti_context_id
            )
            parent_activities.append(
                Activity(
                    id=lti_context.lti_id,
                    definition=ActivityDefinition(
                        type="http://adlnet.gov/expapi/activities/course"
                    ),
                )
            )

        except LTIContext.DoesNotExist:
            pass

    context = Context(
        context_activities=ContextActivities(parent=parent_activities),
    )

    statement = build_statement(user, verb, obj, context)
    if statement:
        xapi_logger.info(statement.to_json())



"""Ashley application"""
from machina.apps.forum_moderation.apps import (
    ForumModerationAppConfig as BaseForumModerationAppConfig,
)


class ForumModerationAppConfig(BaseForumModerationAppConfig):
    """Configuration class for the forum_conversation app."""

    name = "ashley.machina_extensions.forum_moderation"



"""Ashley application"""
from django.apps import AppConfig


class AshleyConfig(AppConfig):
    """Configuration class for the ashley app."""

    name = "ashley"

    def ready(self):
        """Executes whatever is necessary when the application is ready."""
        # pylint: disable=import-outside-toplevel,unused-import
        from . import receivers  # noqa: F401



"""
    Forum views
    ===========

    This module overrides views provided by the ``forum`` application.

"""
from django.db.models import F
from django.http import Http404, HttpResponseRedirect
from django.urls import reverse
from django.utils.http import urlencode
from django.views.generic import UpdateView
from machina.apps.forum.views import ForumView as BaseForumView
from machina.apps.forum.views import IndexView as BaseIndexView
from machina.apps.forum_permission.shortcuts import remove_perm
from machina.apps.forum_permission.viewmixins import (
    PermissionRequiredMixin as BasePermissionRequiredMixin,
)
from machina.core.db.models import get_model
from machina.core.loading import get_class

from ashley.defaults import DEFAULT_FORUM_BASE_WRITE_PERMISSIONS

LTIContext = get_model("ashley", "LTIContext")
Forum = get_model("forum", "Forum")
ForumVisibilityContentTree = get_class("forum.visibility", "ForumVisibilityContentTree")

ORDER_VAR = "o"
PermissionRequiredMixin: BasePermissionRequiredMixin = get_class(
    "forum_permission.viewmixins", "PermissionRequiredMixin"
)


class OrderByColumnMixin:
    """Helper class to order by columns"""

    # default ordering
    default_ordering = "-last_post_on"

    def __init__(self):
        """Init called with default_ordering and list_display"""
        super().__init__()
        self.params = {}

    def get(self, request, *args, **kwargs):
        """Handles GET requests."""
        # collect the current params for the order by in request
        self.params = dict(request.GET.items())
        try:
            super_get = getattr(super(), "get")
            return super_get(request, *args, **kwargs)
        except AttributeError as exception:
            raise RuntimeError(
                "OrderByColumnMixin must be used as part of a "
                "multiple inheritance chain"
            ) from exception

    def get_default_index_order(self):
        """Utility to access index query column and column of default order with sign"""
        field = self.default_ordering.replace("-", "")
        sign = "-" if "-" in self.default_ordering else ""
        return {
            "index": f"{sign}{self.list_display.index(field)}",
            "col": self.default_ordering,
        }

    def get_context_data(self, **kwargs):
        """Returns the context data to provide to the template."""
        try:
            super_context = getattr(super(), "get_context_data")
            context = super_context(**kwargs)
        except AttributeError as exception:
            raise RuntimeError(
                "OrderByColumnMixin must be used as part of a "
                "multiple inheritance chain"
            ) from exception

        # add sort header links containing asc, desc orders
        context["header"] = list(self.result_headers())

        # used for pagination to keep the order, make sure it has expected value
        if self.request.GET.get(ORDER_VAR):
            pfx, idx = self.get_ordering()
            # if value is unexpected, order is ommited
            if f"{pfx}{idx}" == self.request.GET.get(ORDER_VAR):
                context["order"] = f"{ORDER_VAR}={pfx}{idx}"

        return context

    def get_ordering(self):
        """Gets order requested in url"""
        ordering = self.request.GET.get(
            ORDER_VAR, self.get_default_index_order().get("index")
        )
        try:
            _none, pfx, idx = ordering.rpartition("-")
            # returns default column if out of index order
            if int(idx) >= len(self.list_display):
                idx = self.get_default_index_order().get("col")
            return pfx, int(idx)
        except ValueError:
            # param exists but value is unexpected, use default order
            _none, pfx, idx = (
                self.get_default_index_order().get("index").rpartition("-")
            )
            return pfx, int(idx)

    def result_headers(self):
        """Builds each link and css class attribute for the header depending on current order"""
        pfx, idx = self.get_ordering()

        for index, _field_name in enumerate(self.list_display):
            new_order_type = "asc"
            div_classes = ""
            query_string = self.get_query_string({ORDER_VAR: "-" + str(index)})
            # Is it currently being sorted on?
            is_sorted = index == idx
            if is_sorted:
                order_type = "desc" if pfx == "-" else "asc"
                div_classes = f"sorted {order_type}ending"
                new_order_type = {"asc": "desc", "desc": "asc"}[order_type]
                query_string = self.get_query_string(
                    {ORDER_VAR: ("-" if pfx == "" else "") + str(index)}
                )

            yield {
                "sorted": is_sorted,
                "ascending": new_order_type == "desc",
                "url_order": query_string,
                "class_attrib": div_classes,
            }

    def get_query_string(self, new_params):
        """
        Build the query_string for this link keeping all the query params and replacing
        or adding new_params
        """
        param = {**self.params, **new_params}
        query = urlencode(sorted(param.items()))
        return f"?{query}"

    def get_ordering_column(self):
        """From query url, retrieve the concerned field ordered on if any."""
        pfx, idx = self.get_ordering()
        if idx < len(self.list_display) and self.list_display[idx]:
            return pfx + self.list_display[idx]

        return self.get_default_index_order().get("col")


class IndexView(
    OrderByColumnMixin, BaseIndexView
):  # pylint: disable=too-many-ancestors
    """Displays the top-level forums."""

    # header columns
    list_display = ["name", "direct_topics_count", "direct_posts_count", "last_post_on"]

    def get_queryset(self):
        """Returns the list of items for this view ordered by column."""
        pfx, idx = self.get_ordering()
        f_query = F(self.list_display[idx])
        f_query_order_by = (
            f_query.desc(nulls_last=True)
            if pfx == "-"
            else f_query.asc(nulls_last=True)
        )

        return ForumVisibilityContentTree.from_forums(
            self.request.forum_permission_handler.forum_list_filter(
                Forum.objects.filter(archived=False).order_by(f_query_order_by),
                self.request.user,
            ),
        )

    def get_context_data(self, **kwargs):
        """Returns the context data to provide to the template."""
        context = super().get_context_data(**kwargs)
        # Add information about the current lti_context
        try:
            context["course_locked"] = LTIContext.objects.get(
                id=self.request.forum_permission_handler.current_lti_context_id
            ).is_marked_locked
        except LTIContext.DoesNotExist:
            context["course_locked"] = False

        return context


class ForumView(
    OrderByColumnMixin, BaseForumView
):  # pylint: disable=too-many-ancestors
    """Displays a forum and its topics."""

    # header columns
    list_display = ["subject", "posts_count", "views_count", "last_post_on"]

    def get_forum(self):
        """
        Returns the forum to consider and checks that it has not been
        archived.
        """
        forum = super().get_forum()
        if forum.archived:
            raise Http404()
        return forum

    def get_queryset(self):
        """Returns the list of items for this view ordered by asked param."""
        query = super().get_queryset()
        # Type of topic is kept as first order argument to keep sticky option
        return query.order_by("-type", self.get_ordering_column())

    def get_context_data(self, **kwargs):
        context = super().get_context_data()
        # Add information about the current lti_context
        try:
            context["course_locked"] = LTIContext.objects.get(
                id=self.request.forum_permission_handler.current_lti_context_id
            ).is_marked_locked
        except LTIContext.DoesNotExist:
            context["course_locked"] = False

        return context


class ForumArchiveView(PermissionRequiredMixin, UpdateView):
    """Displays the form to archive a forum."""

    model = Forum
    fields = ["archived"]
    template_name = "forum/forum_archive.html"

    def get_success_url(self):
        """Returns the success URL to redirect the user to."""
        return reverse("forum:index")

    # pylint: disable=unused-argument
    def perform_permissions_check(self, user, obj, perms):
        """Performs the permissions check."""
        return self.request.forum_permission_handler.can_archive_forum(obj, user)

    def get_form_kwargs(self):
        """
        Ignores data posted by the user and forces the archiving of the forum.
        """
        kwargs = super().get_form_kwargs()
        kwargs.update({"data": {"archived": True}})
        return kwargs


class ForumLockCourseView(PermissionRequiredMixin, UpdateView):
    """View to lock forums of the course."""

    model = Forum
    fields = ["lti_contexts"]
    template_name = "forum/forum_lock.html"

    def get_forums_list(self):
        """Returns the list of forums of this course"""
        return Forum.objects.filter(
            lti_contexts=self.get_object().lti_contexts.get(
                id=self.request.forum_permission_handler.current_lti_context_id
            )
        )

    def get_context_data(self, **kwargs):
        """Sends forums_list to the view"""
        context = super().get_context_data()
        context["forums_list"] = self.get_forums_list()
        return context

    def get_success_url(self):
        """Returns the success URL to redirect the user to."""
        return reverse("forum:index")

    # pylint: disable=unused-argument
    def perform_permissions_check(self, user, obj, perms):
        """
        Performs the permissions check. If forums is not part of current LTIContext
        access is refused.
        """
        if not obj.lti_contexts.filter(
            id=self.request.forum_permission_handler.current_lti_context_id
        ).exists():
            return False
        return self.request.forum_permission_handler.can_lock_course(obj, user)

    def post(self, request, *args, **kwargs):
        """
        We lock all forums of this course. Default group shouldn't have writing
        permissions anymore. The LTIContex will then be marked as blocked.
        """

        lti_context = self.get_object().lti_contexts.get(
            id=self.request.forum_permission_handler.current_lti_context_id
        )
        default_group = lti_context.get_base_group()

        # remove all permissions for each forum of this LTIContext
        for forum in self.get_forums_list():
            # pylint: disable=not-an-iterable
            for perm in DEFAULT_FORUM_BASE_WRITE_PERMISSIONS:
                remove_perm(perm, default_group, forum)

        # mark this course as locked
        lti_context.is_marked_locked = True
        lti_context.save()

        return HttpResponseRedirect(self.get_success_url())


class ForumRenameView(PermissionRequiredMixin, UpdateView):
    """Allows the current user to update its forum profile."""

    model = Forum
    fields = ["name"]
    template_name = "forum/forum_rename.html"

    def get_success_url(self):
        """Returns the success URL to redirect the user to."""
        return reverse(
            "forum:forum",
            kwargs={
                "pk": self.object.pk,
                "slug": self.object.slug,
            },
        )

    # pylint: disable=unused-argument
    def perform_permissions_check(self, user, obj, perms):
        """Performs the permissions check."""
        return self.request.forum_permission_handler.can_rename_forum(obj, user)



"""
    Forum conversation signals
    ==========================
    This modules defines Django signals that can be triggered by the ``forum_conversation``
    application.
"""

import django.dispatch

topic_created = django.dispatch.Signal(
    providing_args=["topic", "user", "request", "response"]
)
topic_updated = django.dispatch.Signal(
    providing_args=["topic", "user", "request", "response"]
)
post_created = django.dispatch.Signal(
    providing_args=["post", "user", "request", "response"]
)
post_updated = django.dispatch.Signal(
    providing_args=["post", "user", "request", "response"]
)



"""
    Forum search forms
    ==================
    This module defines forms provided by the ``forum_search`` application.
"""
from django.utils.translation import gettext_lazy as _
from machina.apps.forum_search.forms import SearchForm as MachinaSearchForm
from machina.core.db.models import get_model
from machina.core.loading import get_class

Forum = get_model("forum", "Forum")
PermissionHandler = get_class("forum_permission.handler", "PermissionHandler")


class SearchForm(MachinaSearchForm):
    """
    Allows to search forum topics and posts and allows searching poster user names
    even with an empty search in the main field.
    """

    def __init__(self, *args, **kwargs):
        user = kwargs.pop("user", None)
        lti_contexts = kwargs.pop("lti_context", None)

        # pylint: disable=bad-super-call
        super(MachinaSearchForm, self).__init__(*args, **kwargs)

        # Update some fields
        self.fields["q"].label = _("Search for keywords")
        self.fields["q"].widget.attrs["placeholder"] = _("Keywords or phrase")
        self.fields["search_poster_name"].widget.attrs["placeholder"] = _("Poster name")

        self.perm_handler = PermissionHandler()

        # add context
        if lti_contexts:
            self.perm_handler.current_lti_context_id = lti_contexts.id

        # forums gets filtered by lti_context and exclude archives directly
        # in get_readable_forums
        self.allowed_forums = self.perm_handler.get_readable_forums(
            Forum.objects.all(), user
        )
        # pylint: disable=consider-using-f-string
        if self.allowed_forums:
            self.fields["search_forums"].choices = [
                (f.id, "{} {}".format("-" * f.margin_level, f.name))
                for f in self.allowed_forums
            ]
        else:
            # The user cannot view any single forum, the 'search_forums' field can be deleted
            del self.fields["search_forums"]

    def clean(self):
        """
        Set main query to catch all "*" if it is empty and there is a search term for
        poster username.
        """
        cleaned_data = super().clean()

        if cleaned_data.get("search_poster_name") and not cleaned_data.get("q"):
            cleaned_data["q"] = "*"

        return cleaned_data



"""Ashley template tags to display forum content"""
from django import template
from machina.core.db.models import get_model
from machina.templatetags.forum_tags import forum_list

from ashley.defaults import _FORUM_ROLE_ADMINISTRATOR, _FORUM_ROLE_INSTRUCTOR

LTIContext = get_model("ashley", "LTIContext")

register = template.Library()


@register.filter()
def is_user_instructor(topic, user):
    """
    This will return a boolean indicating if the passed user is instructor
    for the given topic.
    Usage::
        {% if topic|is_user_instructor:user %}...{% endif %}
    """
    # get the list of lti concerns by this forum, one forum can have multiple LTI Context
    # if the user is instructor in one of them then he is considered
    # as instructor for this forum
    return any(
        _FORUM_ROLE_INSTRUCTOR in lti.get_user_roles(user)
        for lti in LTIContext.objects.filter(forum=topic.forum)
    )


@register.filter()
def is_user_administrator(topic, user):
    """
    This will return a boolean indicating if the passed user is admin
    for the given topic.
    Usage::
        {% if topic|is_user_administrator:user %}...{% endif %}
    """
    # get the list of lti concerns by this forum, one forum can have multiple LTI Context
    # if the user is admin in one of them then he is considered
    # as admin for this forum
    return any(
        _FORUM_ROLE_ADMINISTRATOR in lti.get_user_roles(user)
        for lti in LTIContext.objects.filter(forum=topic.forum)
    )


@register.inclusion_tag("forum/forum_list.html", takes_context=True)
def forum_list_order(context, forum_visibility_contents):
    """Renders the considered forum list with the column order added.

    Usage::

        {% forum_list_order my_forums %}

    """
    data_dict = forum_list(context, forum_visibility_contents)
    data_dict["header"] = context.get("header")
    return data_dict



"""
    Forum moderation forms
    ======================

    This module overrides forms provided by the ``forum_moderation`` application.

"""
from machina.apps.forum_moderation.forms import TopicMoveForm as MachinaTopicMoveForm
from machina.core.db.models import get_model
from machina.core.loading import get_class

Forum = get_model("forum", "Forum")
PermissionHandler = get_class("forum_permission.handler", "PermissionHandler")


class TopicMoveForm(MachinaTopicMoveForm):
    """Allows to move a topic."""

    def __init__(self, *args, **kwargs):
        """
        Overrides original __init__ method to filter forums in order to show only the ones from
        the same LTIContext
        """
        lti_context = kwargs.pop("lti_context", None)
        super().__init__(*args, **kwargs)
        # Overrides method and instanciates PermissionHandler with current lti_context.id
        self.perm_handler = PermissionHandler()
        self.perm_handler.current_lti_context_id = lti_context.id
        self.allowed_forums = self.perm_handler.get_target_forums_for_moved_topics(
            self.user
        )

        allowed_forums_ids = [f.id for f in self.allowed_forums]
        filtered_choices = filter(
            lambda c: c[0] in allowed_forums_ids, self.fields["forum"].choices
        )
        self.fields["forum"].choices = filtered_choices



"""
    Forum conversation views
    ========================
    This module defines views provided by the ``forum_conversation`` application.
"""

from machina.apps.forum_conversation.views import PostCreateView as BasePostCreateView
from machina.apps.forum_conversation.views import PostUpdateView as BasePostUpdateView
from machina.apps.forum_conversation.views import TopicCreateView as BaseTopicCreateView
from machina.apps.forum_conversation.views import TopicUpdateView as BaseTopicUpdateView
from machina.apps.forum_conversation.views import TopicView as BaseTopicView
from machina.core.db.models import get_model

from .signals import post_created, post_updated, topic_created, topic_updated

LTIContext = get_model("ashley", "LTIContext")
Topic = get_model("forum_conversation", "Topic")


class SendSignalMixin:
    """Implements the signal sending at the view level."""

    view_signal = None

    def send_signal(self, request, response, **kwargs):
        """Sends the signal associated with the view."""

        self.view_signal.send(
            sender=self,
            user=request.user,
            request=request,
            response=response,
            **kwargs,
        )


class TopicCreateView(SendSignalMixin, BaseTopicCreateView):
    """Creates a topic within a forum."""

    view_signal = topic_created

    def form_valid(self, post_form, attachment_formset, poll_option_formset, **kwargs):
        """Raises topic created signal on successful topic creation"""

        response = super().form_valid(
            post_form, attachment_formset, poll_option_formset, **kwargs
        )
        self.send_signal(self.request, response, topic=self.forum_post.topic)

        return response


class TopicUpdateView(SendSignalMixin, BaseTopicUpdateView):
    """Updates an existing topic."""

    view_signal = topic_updated

    def form_valid(self, post_form, attachment_formset, poll_option_formset, **kwargs):
        """Raises topic updated signal on successful topic update"""

        response = super().form_valid(
            post_form, attachment_formset, poll_option_formset, **kwargs
        )
        self.send_signal(self.request, response, topic=self.forum_post.topic)

        return response


class PostCreateView(SendSignalMixin, BasePostCreateView):
    """Creates a post within a topic."""

    view_signal = post_created

    def form_valid(self, post_form, attachment_formset, **kwargs):
        """Raises post created signal on successful post creation"""

        response = super().form_valid(post_form, attachment_formset, **kwargs)

        self.send_signal(self.request, response, post=self.forum_post)

        return response


class PostUpdateView(SendSignalMixin, BasePostUpdateView):
    """Creates a post within a topic."""

    view_signal = post_updated

    def form_valid(self, post_form, attachment_formset, **kwargs):
        """Raises post created signal on successful post creation"""

        response = super().form_valid(post_form, attachment_formset, **kwargs)

        self.send_signal(self.request, response, post=self.forum_post)

        return response


# pylint: disable=too-many-ancestors
class TopicView(BaseTopicView):
    """Displays a forum topic."""

    def get_context_data(self, **kwargs):
        """Returns the context data to provide to the template."""
        context = super().get_context_data(**kwargs)
        # Add information about the current lti_context
        try:
            context["course_locked"] = LTIContext.objects.get(
                id=self.request.forum_permission_handler.current_lti_context_id
            ).is_marked_locked
        except LTIContext.DoesNotExist:
            context["course_locked"] = False

        return context



# Generated by Django 3.2.4 on 2021-07-13 13:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("forum", "0015_auto_20210217_1351"),
    ]

    operations = [
        migrations.AddField(
            model_name="forum",
            name="archived",
            field=models.BooleanField(default=False),
        ),
    ]



"""
    Forum moderation views
    ======================

    This module overrides views provided by the ``forum_moderation`` application.

"""

from machina.apps.forum_moderation.views import TopicMoveView as MachinaTopicMoveView

from ashley.context_mixins import get_current_lti_session


class TopicMoveView(MachinaTopicMoveView):
    """Overrides MachinaTopicMoveView to filter on forums part of the LTIContext"""

    def get_form_kwargs(self):
        """Returns the keyword arguments used to initialize the associated form."""
        return {
            **super().get_form_kwargs(),
            "lti_context": get_current_lti_session(self.request),
        }



"""Admin of the Ashley application."""
from django.contrib import admin

from .models import User


@admin.register(User)
class UserAdmin(admin.ModelAdmin):
    """Admin class for Ashley's User model."""

    search_fields = (
        "username",
        "first_name",
        "last_name",
        "email",
        "public_username",
        "lti_remote_user_id",
    )
